package de.thefeiter.liedgutverzeichnis

import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.app.AppCompatDelegate
import androidx.appcompat.widget.Toolbar
import androidx.recyclerview.widget.RecyclerView
import androidx.room.Room
import androidx.viewpager2.widget.ViewPager2
import com.google.android.material.tabs.TabLayout
import com.google.android.material.tabs.TabLayoutMediator
import de.thefeiter.Liedgutverzeichnis
import de.thefeiter.liedgutverzeichnis.api.ApiTest
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.api.updater.AUTO_SYNC_INTERVAL
import de.thefeiter.liedgutverzeichnis.api.updater.UpdaterAll
import de.thefeiter.liedgutverzeichnis.api.uploader.UploaderSuggestions
import de.thefeiter.liedgutverzeichnis.databinding.ActivityMainBinding
import de.thefeiter.liedgutverzeichnis.helpers.SnackbarHelper
import de.thefeiter.liedgutverzeichnis.helpers.SyncHelper
import de.thefeiter.liedgutverzeichnis.helpers.TopFlopNotificator
import de.thefeiter.liedgutverzeichnis.objects.DEFAULT_TOKEN_VALUE
import de.thefeiter.liedgutverzeichnis.objects.User
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.dialogs.UserSetupDialog
import de.thefeiter.liedgutverzeichnis.ui.main.SectionsPagerAdapter
import de.thefeiter.liedgutverzeichnis.ui.settings.PREFERENCE_NAME
import de.thefeiter.liedgutverzeichnis.ui.settings.SHARED_PREFS_DARK_MODE
import de.thefeiter.liedgutverzeichnis.ui.settings.SHARED_PREFS_LAST_DB_SYNC
import de.thefeiter.liedgutverzeichnis.ui.settings.SettingsActivity
import de.thefeiter.liedgutverzeichnis.ui.songdetails.LAST_SONG_PREF
import de.thefeiter.liedgutverzeichnis.ui.songdetails.LAST_SONG_PREF_SONG_ID
import de.thefeiter.liedgutverzeichnis.ui.songdetails.SongDetails
import de.thefeiter.liedgutverzeichnis.ui.topflop.CatTopFlopActivity
import de.thefeiter.liedgutverzeichnis.ui.topflop.SongTopFlopActivity


class MainActivity : AppCompatActivity() {
    private val appDb = getAppDb()
    private lateinit var lastSongBar: RelativeLayout
    private lateinit var cacheUploadIcon: ImageView
    private lateinit var syncIcon: ImageView

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        lastSongBar = findViewById(R.id.last_song_bar)
        syncIcon = findViewById(R.id.syncIcon)
        cacheUploadIcon = findViewById(R.id.cacheUploadIcon)
        val myToolbar = findViewById<Toolbar>(R.id.my_toolbar)
        setSupportActionBar(myToolbar)

        val sectionsPagerAdapter = SectionsPagerAdapter(this, supportFragmentManager, lifecycle)
        val viewPager = binding.viewPager

        val sharedPreferences = getSharedPreferences(PREFERENCE_NAME, MODE_PRIVATE)

        //adjust touch slop so that tabs don't scroll as easy
        try {
            val recyclerViewField = ViewPager2::class.java.getDeclaredField("mRecyclerView")
            recyclerViewField.isAccessible = true
            val recyclerView = recyclerViewField[viewPager] as RecyclerView
            val touchSlopField = RecyclerView::class.java.getDeclaredField("mTouchSlop")
            touchSlopField.isAccessible = true
            val touchSlopFieldObject = touchSlopField[recyclerView]
            if (touchSlopFieldObject != null) {
                val touchSlop = touchSlopFieldObject as Int
                touchSlopField[recyclerView] = touchSlop * 3 //3 is empirical value
            }
        } catch (ignore: Exception) {
        }

        viewPager.adapter = sectionsPagerAdapter

        //tab init
        val tabs = binding.tabs
        tabs.tabMode = TabLayout.MODE_SCROLLABLE

        TabLayoutMediator(
            tabs, viewPager
        ) { tab: TabLayout.Tab, position: Int ->
            //tab.setText(sectionsPagerAdapter.titles[position])
            tab.setIcon(sectionsPagerAdapter.icons[position])
        }.attach()


        //random fab
        val fab = binding.fab
        fab.setOnClickListener {
            val randomSong = appDb.songDao().loadRandomFilteredByUser()

            //if random song fails, notify and return
            randomSong?.let {
                //start new activity with song object by intent
                val intent = Intent(this, SongDetails::class.java)
                intent.putExtra(SONG_DETAILS, randomSong.id)
                intent.putExtra(BY_RANDOM, true)
                startActivity(intent)
            } ?: run {
                SnackbarHelper(findViewById(R.id.mainCoordLayout)).makeNoBookSnackbar()
            }
        }

        //apply darkmode
        val darkModeValue = sharedPreferences.getInt(
            SHARED_PREFS_DARK_MODE,
            AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM
        )
        AppCompatDelegate.setDefaultNightMode(darkModeValue)


        // check Database if User exists
        if (User().apiToken == DEFAULT_TOKEN_VALUE) {
            //register oder enter api key
            val setupDialog = UserSetupDialog(
                appDb,
                this,
                supportFragmentManager,
                SyncHelper(syncIcon),
                findViewById(R.id.mainCoordLayout),
                true
            )
            setupDialog.isCancelable = false
            setupDialog.show()


        } else {
            //check for new topflop submissions
            createNotificationChannel()
            TopFlopNotificator(this, appDb).checkAndNotify()
            checkLastDbSync(sharedPreferences)
        }

        //sync onclick
        syncIcon.setOnClickListener {
            updateDatabase()
        }

        //cache upload onclick
        cacheUploadIcon.setOnClickListener {
            val sh = SyncHelper(cacheUploadIcon)
            sh.setUploadingState(true)
            UploaderSuggestions(this, appDb) {
                sh.setUploadingState(false)
                cacheUploadIcon.visibility = View.GONE
                SnackbarHelper(findViewById(R.id.mainCoordLayout)).makeSnackbar(getString(R.string.cache_uploaded))
            }.testAndUpload()
        }


    }

    private fun checkLastDbSync(sharedPrefs: SharedPreferences) {
        val lastDbSyncTs = sharedPrefs.getInt(SHARED_PREFS_LAST_DB_SYNC, 0)
        if (timestamp - lastDbSyncTs >= AUTO_SYNC_INTERVAL) {
            updateDatabase()
        }
    }

    private fun updateDatabase() {
        UpdaterAll(
            this,
            SyncHelper(syncIcon),
            findViewById(R.id.dbUpdateProgress),
            { SnackbarHelper(findViewById(R.id.mainCoordLayout)).makeSnackbar(getString(R.string.updating_db)) }
        ) {
            SnackbarHelper(findViewById(R.id.mainCoordLayout)).makeSnackbar(getString(R.string.updating_db_done))
        }.update(appDb)
    }

    override fun onResume() {
        super.onResume()

        val lastSongPref = getSharedPreferences(LAST_SONG_PREF, MODE_PRIVATE)
        val lastSongId = lastSongPref.getInt(
            LAST_SONG_PREF_SONG_ID,
            0
        )

        val lastSong = appDb.songDao().findById(lastSongId)

        lastSong?.let { song ->
            lastSongBar.visibility = View.VISIBLE
            lastSongBar.findViewById<TextView>(R.id.last_song_title).text = song.title
            lastSongBar.findViewById<TextView>(R.id.last_song_beginning).text = song.beginning
            lastSongBar.findViewById<TextView>(R.id.last_song_beginning)
                .setOnClickListener { view ->

                    //start new activity with song object by intent
                    val intent = Intent(view.context, SongDetails::class.java)
                    intent.putExtra(SONG_DETAILS, song.id)

                    view.context.startActivity(intent)
                }

        }

        appDb.suggestionCacheDao().getFirst()?.let {
            cacheUploadIcon.visibility = View.VISIBLE
        } ?: run {
            cacheUploadIcon.visibility = View.INVISIBLE
        }

    }

    override fun onPrepareOptionsMenu(menu: Menu?): Boolean {
        super.onPrepareOptionsMenu(menu)

        //add topflop button to menu if superuser
        val user = User()
        menu?.findItem(R.id.catTopflop)?.isVisible = user.allowTopflop
        menu?.findItem(R.id.songTopflop)?.isVisible = user.allowTopflop
        menu?.findItem(R.id.resetTopflop)?.isVisible = user.allowTopflop
        return true
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        val inflater = menuInflater
        inflater.inflate(R.menu.menu_main, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // Handle item selection
        return when (item.itemId) {
            R.id.settings -> {
                //start new activity with song object by intent
                val intent = Intent(this, SettingsActivity::class.java)
                startActivity(intent)
                true
            }
            R.id.feedback -> {

                val i = Intent(Intent.ACTION_VIEW)
                i.data = Uri.parse(getString(R.string.git_issue_url))
                startActivity(i)
                true

            }
            R.id.help -> {
                //start new activity with song object by intent
                val intent = Intent(this, HelpActivity::class.java)
                startActivity(intent)
                true
            }
            R.id.visuals -> {

                val i = Intent(Intent.ACTION_VIEW)
                i.data = Uri.parse(getString(R.string.api_visuals_url))
                startActivity(i)
                true

            }
            R.id.catTopflop -> {

                val intent = Intent(this, CatTopFlopActivity::class.java)
                startActivity(intent)
                true

            }
            R.id.songTopflop -> {

                val intent = Intent(this, SongTopFlopActivity::class.java)
                startActivity(intent)
                true

            }
            R.id.resetTopflop -> {

                val apiTest = ApiTest(this)
                apiTest.test(appDb) {
                    val ld = LoaderArray(
                        this,
                        appDb,
                        {},
                        {
                            SnackbarHelper(findViewById(R.id.mainCoordLayout)).makeSnackbar(
                                getString(
                                    R.string.success
                                )
                            )
                        },
                        getString(R.string.api_base_url) +
                                getString(R.string.endpoint_topflop_category_action_prefix) +
                                getString(R.string.endpoint_topflop_action_reset)
                    )

                    LoaderArray(
                        this,
                        appDb,
                        {},
                        { ld.load() },
                        getString(R.string.api_base_url) +
                                getString(R.string.endpoint_topflop_song_action_prefix) +
                                getString(R.string.endpoint_topflop_action_reset)
                    ).load()
                }
                true

            }
            else -> {
                super.onOptionsItemSelected(item)
            }
        }
    }

    private fun createNotificationChannel() {
        // Create the NotificationChannel, but only on API 26+ because
        // the NotificationChannel class is new and not in the support library
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O && User().allowTopflop) {
            val name = getString(R.string.notificationChannel_topflop)
            val descriptionText = getString(R.string.notificationChannel_topflop_description)
            val importance = NotificationManager.IMPORTANCE_DEFAULT
            val channel =
                NotificationChannel(CHANNEL_ID_TOPFLOP_NOTIFICATION, name, importance).apply {
                    description = descriptionText
                }
            // Register the channel with the system
            val notificationManager: NotificationManager =
                getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
            notificationManager.createNotificationChannel(channel)
        }
    }

    companion object {
        const val SONG_DETAILS = "de.thefeiter.liedgutverzeichnis.SONG_DETAILS"
        const val CHANNEL_ID_TOPFLOP_NOTIFICATION =
            "de.thefeiter.liedgutverzeichnis.TOPFLOP_NOTIFICATIONS"

        @JvmStatic
        fun getAppDb(): AppDb {
            val context = Liedgutverzeichnis.getAppContext()
            return Room.databaseBuilder(
                context,
                AppDb::class.java,
                context.getString(R.string.roomDatabaseName)
            )
                .addMigrations(AppDb.MIGRATION_32_33, AppDb.MIGRATION_34_35)
                .allowMainThreadQueries().build() //TODO disallow main thread queries
        }

        @JvmStatic
        val timestamp: Double
            get() = System.currentTimeMillis() / 1000.0
    }
}