package de.thefeiter.liedgutverzeichnis.api

import android.content.Context
import android.net.ConnectivityManager
import android.net.NetworkCapabilities
import android.widget.Toast
import androidx.fragment.app.FragmentActivity
import com.android.volley.Request
import com.android.volley.Response
import com.android.volley.VolleyError
import com.android.volley.toolbox.JsonObjectRequest
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.timestamp
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.helpers.SyncHelper
import de.thefeiter.liedgutverzeichnis.objects.User
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.dialogs.UserSetupDialog

const val AUTH_TTL = 24 * 3600

class ApiTest(private val context: Context, private val silent: Boolean) {
    constructor(context: Context) : this(context, false)

    val user = User()

    fun networkUnreachable(): Boolean {
        val manager = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val capabilities = manager.getNetworkCapabilities(manager.activeNetwork)

        return !(capabilities != null && capabilities.hasCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET))
    }

    fun test(appDb: AppDb, onSuccess: () -> Unit) {
        val testToken = user.apiToken
        test(appDb, onSuccess, {}, testToken)
    }

    fun test(appDb: AppDb, onSuccess: () -> Unit, onFail: () -> Unit) {
        val testToken = user.apiToken
        test(appDb, onSuccess, onFail, testToken)
    }

    fun test(appDb: AppDb, onSuccess: () -> Unit, testToken: String) {
        test(appDb, onSuccess, {}, testToken)
    }

    fun test(appDb: AppDb, onSuccess: () -> Unit, onFail: () -> Unit, testToken: String) {

        // notify about no network
        if (networkUnreachable()) {
            if (!silent) Toast.makeText(context, R.string.error_no_network, Toast.LENGTH_LONG)
                .show()
            onFail.invoke()
            return
        }


        val authRequest: JsonObjectRequest = object : JsonObjectRequest(
            Method.GET,
            context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_auth_check),
            null,
            Response.Listener {
                //on request success execute callback
                user.lastTokenCheck = timestamp.toLong()
                onSuccess.invoke()
            },
            Response.ErrorListener { error: VolleyError ->
                //on error notify about error
                if (error.networkResponse.statusCode == 401) {
                    if (!silent) Toast.makeText(
                        context,
                        R.string.error_token_invalid,
                        Toast.LENGTH_LONG
                    ).show()
                    onFail.invoke()
                    showSetupDialog(appDb)
                }
            }) {
            /**
             * Passing some request headers
             */
            override fun getHeaders(): Map<String, String> {
                val headers = HashMap<String, String>()
                headers[context.getString(R.string.api_headers_token)] = testToken
                return headers
            }
        }
        val pingRequest = JsonObjectRequest(
            Request.Method.GET,
            context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_ping),
            null,
            {
                //if api responds, add auth check request
                if (timestamp - user.lastTokenCheck < AUTH_TTL) {
                    //skip auth check
                    onSuccess.invoke()
                } else {
                    MySingleton.getInstance(context).addToRequestQueue(authRequest)
                }
            }
        ) { error: VolleyError ->
            //on error notify about error
            error.printStackTrace()
            if (!silent) Toast.makeText(context, R.string.error_api_unreachable, Toast.LENGTH_LONG)
                .show()
            onFail.invoke()
        }


        // Access the RequestQueue through your singleton class.
        MySingleton.getInstance(context).addToRequestQueue(pingRequest)
    }

    private fun showSetupDialog(appDb: AppDb) {
        try {
            // try to reshow setup dialog
            val fragmentManager = (context as FragmentActivity).supportFragmentManager
            val setupDialog = UserSetupDialog(
                appDb,
                context,
                fragmentManager,
                SyncHelper(context.findViewById(R.id.syncIcon))
            )
            setupDialog.show()
        } catch (e: ClassCastException) {
            e.printStackTrace()
        }
    }
}