package de.thefeiter.liedgutverzeichnis.ui.settings

import android.content.ClipData
import android.content.ClipboardManager
import android.content.SharedPreferences
import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.app.AppCompatDelegate
import androidx.appcompat.widget.Toolbar
import androidx.preference.ListPreference
import androidx.preference.Preference
import androidx.preference.PreferenceDataStore
import androidx.preference.PreferenceFragmentCompat
import de.thefeiter.liedgutverzeichnis.MainActivity
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.helpers.SyncHelper
import de.thefeiter.liedgutverzeichnis.objects.User
import de.thefeiter.liedgutverzeichnis.ui.dialogs.BookSelectionDialog
import de.thefeiter.liedgutverzeichnis.ui.dialogs.UserSetupDialog


const val PREFERENCE_NAME = "main_settings_sharedPref"
const val SHARED_PREFS_DARK_MODE = "DarkMode"
const val SHARED_PREFS_LAST_NOTIFICATION = "LastNotification"
const val SHARED_PREFS_USER_API_TOKEN = "UserApiToken"
const val SHARED_PREFS_USER_SUPERUSER = "UserSuperuser"
const val SHARED_PREFS_USER_TOPFLOP_ALLOWED = "UserTopflopAllowed"
const val SHARED_PREFS_USER_LAST_AUTH_CHECK = "UserLastAuthCheck"
const val SHARED_PREFS_LAST_DB_SYNC = "LastDatabaseSync"


class SettingsActivity : AppCompatActivity() {

    private lateinit var fragment: SettingsFragment

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_settings)

        if (savedInstanceState == null) {
            fragment =
                SettingsFragment(SyncHelper(findViewById(R.id.syncIcon))) // Empty constructor!
            this.supportFragmentManager
                .beginTransaction()
                .replace(R.id.settings_layout, fragment)
                .commit()
        }

        //adding the back button functionality
        // songChooseToolbar is defined in the layout file
        val myChildToolbar = findViewById<Toolbar>(R.id.settingsToolbar)
        setSupportActionBar(myChildToolbar)
        // Get a support ActionBar corresponding to this toolbar and enable the Up button
        supportActionBar!!.setDisplayHomeAsUpEnabled(true)


    }

    override fun onDestroy() {
        super.onDestroy()

        val sharedPreferences: SharedPreferences =
            this.getSharedPreferences(PREFERENCE_NAME, MODE_PRIVATE)
        val darkModeValue = sharedPreferences.getInt(
            SHARED_PREFS_DARK_MODE,
            AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM
        )
        AppCompatDelegate.setDefaultNightMode(darkModeValue)
    }

    class SettingsFragment(val syncHelper: SyncHelper) : PreferenceFragmentCompat() {
        private val db = MainActivity.getAppDb()

        override fun onCreatePreferences(savedInstanceState: Bundle?, rootKey: String?) {
            setPreferencesFromResource(R.xml.main_preferences, rootKey)
            val sharedPreferences: SharedPreferences =
                requireContext().getSharedPreferences(PREFERENCE_NAME, MODE_PRIVATE)

            //Book settings
            val bookSettingsPrefId = getString(R.string.settings_bookSettings_chooseBooks_key)
            val bookSettingsPref = findPreference<Preference>(bookSettingsPrefId)
            bookSettingsPref?.setSummary(R.string.settings_bookSettings_chooseBooks_summary)
            bookSettingsPref?.onPreferenceClickListener =
                Preference.OnPreferenceClickListener {
                    val bookSelectionDialog =
                        BookSelectionDialog(MainActivity.getAppDb(), requireContext())
                    bookSelectionDialog.show(parentFragmentManager, null)
                    false
                }

            // Dark mode
            val darkModePrefId =
                context?.getString(R.string.settings_appearance_dark_mode_key) ?: return
            val darkMode: ListPreference? = findPreference(darkModePrefId)
            darkMode?.setSummary(R.string.settings_appearance_dark_mode_summary)
            darkMode?.value = sharedPreferences.getInt(
                SHARED_PREFS_DARK_MODE,
                AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM
            ).toString()
            darkMode?.preferenceDataStore = object : PreferenceDataStore() {
                override fun putString(key: String?, value: String?) {
                    val darkModeValue = value?.toIntOrNull() ?: return
                    if (darkModeValue == AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM) {
                        sharedPreferences.edit()
                            .remove(SHARED_PREFS_DARK_MODE)
                            .apply()
                    } else {
                        sharedPreferences.edit()
                            .putInt(SHARED_PREFS_DARK_MODE, darkModeValue)
                            .apply()
                    }
                    //AppCompatDelegate.setDefaultNightMode(darkModeValue)

                }

                override fun getString(key: String?, defValue: String?): String {
                    return sharedPreferences.getInt(
                        SHARED_PREFS_DARK_MODE,
                        AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM
                    ).toString()
                }
            }

            //Api
            //Copy Token
            val copyTokenPrefId =
                context?.getString(R.string.settings_api_copy_token_key) ?: return
            val copyToken: Preference? = findPreference(copyTokenPrefId)
            copyToken?.setSummary(R.string.settings_api_copy_token_summary)
            copyToken?.onPreferenceClickListener =
                Preference.OnPreferenceClickListener {
                    val clipboard: ClipboardManager =
                        context?.getSystemService(CLIPBOARD_SERVICE) as ClipboardManager

                    val token = User().apiToken

                    val clip: ClipData =
                        ClipData.newPlainText(context?.getString(R.string.api_token), token)
                    clipboard.setPrimaryClip(clip)
                    Toast.makeText(context, getString(R.string.copied), Toast.LENGTH_SHORT).show()
                    false
                }


            //Change Token
            val changeTokenPrefId =
                context?.getString(R.string.settings_api_change_token_key) ?: return
            val changeToken: Preference? = findPreference(changeTokenPrefId)
            changeToken?.setSummary(R.string.settings_api_change_token_summary)
            changeToken?.onPreferenceClickListener =
                Preference.OnPreferenceClickListener {
                    val setupDialog =
                        UserSetupDialog(
                            db, requireContext(), parentFragmentManager,
                            syncHelper
                        )
                    setupDialog.show()
                    false
                }


        }
    }


}