package de.thefeiter.liedgutverzeichnis.ui.songdetails

import android.os.Bundle
import android.view.MenuItem
import android.view.View
import android.widget.ImageView
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.widget.Toolbar
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.chip.ChipGroup
import de.thefeiter.liedgutverzeichnis.BOOK_ID
import de.thefeiter.liedgutverzeichnis.BY_RANDOM
import de.thefeiter.liedgutverzeichnis.MainActivity
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.getAppDb
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.timestamp
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderPdf
import de.thefeiter.liedgutverzeichnis.helpers.SnackbarHelper
import de.thefeiter.liedgutverzeichnis.helpers.SongPathHelper
import de.thefeiter.liedgutverzeichnis.objects.Book
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.objects.User
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.dialogs.songpathselection.SongPathSelectionDialog
import de.thefeiter.liedgutverzeichnis.ui.dialogs.songsequencesuggestion.SongSequenceSuggestionDialog
import de.thefeiter.liedgutverzeichnis.ui.views.Button

const val LAST_SONG_PREF = "SongDetails.LastSongPreference"
const val LAST_SONG_PREF_SONG_ID = "SongDetails.LastSongPreference.SongId"

class SongDetails : AppCompatActivity() {

    private var user: User = User()
    private var db: AppDb = getAppDb()
    private var song: Song? = null
    private var lastSong: Song? = null
    private var book: Book? = null
    private lateinit var pageRecyclerView: RecyclerView
    private lateinit var catChipGroup: ChipGroup
    private lateinit var suggestionChipGroup: ChipGroup
    private lateinit var toolbar: Toolbar
    private var initialSong: Boolean = true
    private var songOpenedByRandom: Boolean = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_song_details)

        pageRecyclerView = findViewById(R.id.details_bookPages_recycler)
        catChipGroup = findViewById(R.id.details_category_chips)
        suggestionChipGroup = findViewById(R.id.details_suggestion_chips)
        toolbar = findViewById(R.id.songDetailsToolbar)

        //adding the back button functionality
        // songChooseToolbar is defined in the layout file
        setSupportActionBar(toolbar)
        // Get a support ActionBar corresponding to this toolbar and enable the Up button
        supportActionBar!!.setDisplayHomeAsUpEnabled(true)


        //set pages recycler view
        pageRecyclerView.layoutManager = LinearLayoutManager(
            this,
            LinearLayoutManager.VERTICAL,
            false
        )

        // set random status
        songOpenedByRandom = intent.getBooleanExtra(BY_RANDOM, false)


        // load book if available
        val bookIdSerializable = intent.getSerializableExtra(BOOK_ID)
        if (bookIdSerializable != null) {
            val bookId = bookIdSerializable as Int
            book = getAppDb().bookDao().findById(bookId)
        }


        // set buttons and button actions
        val newRandomSong = findViewById<View>(R.id.details_btn_random)
        newRandomSong.setOnClickListener { randFabClick() }
        val favoriteBtn = findViewById<View>(R.id.details_btn_favorite)
        favoriteBtn.setOnClickListener {
            song?.let {
                db.songDao().toggleFavorite(it.id, timestamp)
                updateFavButton()
            }

        }
        val bookmarkBtn = findViewById<ImageView>(R.id.details_btn_bookmark)
        bookmarkBtn.setOnClickListener {
            song?.let {
                db.songDao().toggleBookmark(it.id, timestamp)
                updateBookmarkButton()
            }

        }
        val pathBtn = findViewById<ImageView>(R.id.details_btn_find_path)
        pathBtn.setOnClickListener {
            song?.let {
                val bookmarks = db.songDao().getAllBookmarks()
                val paths = SongPathHelper(db).findSongPath(bookmarks, it)

                if (paths.isEmpty()) {
                    val snackbarHelper = SnackbarHelper(findViewById(R.id.songDetails_main_layout))
                    snackbarHelper.makeSnackbar(getString(R.string.songpath_no_path_found))
                } else {
                    val dialog = SongPathSelectionDialog(db, this, paths) {
                        updateUpcomingSongButton()
                    }

                    dialog.show(supportFragmentManager, null)
                }

            }

        }

        // Get the Intent that started this activity and extract the string
        song = db.songDao().findById(intent.getSerializableExtra(MainActivity.SONG_DETAILS) as Int)
        updateView()
        initialSong = true
    }

    private fun updateView() {
        song?.let { song ->
            //set last opened song for last song bar
            val lastSongPref = getSharedPreferences(LAST_SONG_PREF, MODE_PRIVATE)
            lastSongPref.edit().putInt(LAST_SONG_PREF_SONG_ID, song.id).apply()
        }

        lastSong = db.historyDao().lastAsSong
        initialSong = false
        if (!songOpenedByRandom) appendToHistory()
        updateTexts()
        updateFavButton()
        updateBookmarkButton()
        updatePdfButton()
        updateSuggestButton()
        updateCategoryPills()
        updateSuggestionPills()
        updatePages()
        updateUpcomingSongButton()
    }

    private fun updateFavButton() {
        song?.let { song ->
            val favoriteBtnIcon = findViewById<ImageView>(R.id.details_btn_favorite)
            if (db.songDao().isFavorite(song.id)) {
                favoriteBtnIcon.setImageResource(R.drawable.ic_baseline_star_black_24dp)
            } else {
                favoriteBtnIcon.setImageResource(R.drawable.ic_baseline_star_border_black_24dp)
            }
        }

    }

    private fun updateBookmarkButton() {
        song?.let { song ->
            val bookmarkBtn = findViewById<ImageView>(R.id.details_btn_bookmark)
            if (db.songDao().isBookmark(song.id)) {
                bookmarkBtn.setImageResource(R.drawable.ic_baseline_bookmark_remove_black_24dp)
            } else {
                bookmarkBtn.setImageResource(R.drawable.ic_outline_bookmark_add_black_24dp)
            }
        }

    }

    private fun updatePdfButton() {
        song?.let { song ->
            val allowed = user.superuser && song.pdfId != 0
            val pdfBtn = findViewById<View>(R.id.details_btn_pdf)
            pdfBtn.visibility = if (allowed) View.VISIBLE else View.GONE
            pdfBtn.setOnClickListener {
                val loader = LoaderPdf(song, db, this) {
                    pdfBtn.alpha = 1F
                }
                pdfBtn.alpha = 0.5F
                loader.loadPdf()
            }
        }

    }

    private fun updateSuggestButton() {
        song?.let { song ->
            val suggestBtn = findViewById<View>(R.id.details_btn_suggest_from_last)

            lastSong?.let { lastSong ->
                suggestBtn.visibility = View.VISIBLE
                suggestBtn.setOnClickListener {

                    val sequence = ArrayList<Song>()
                    sequence.add(lastSong)
                    sequence.add(song)
                    try {
                        val sequenceSuggestionDialog =
                            SongSequenceSuggestionDialog(db, sequence, this)
                        sequenceSuggestionDialog.show(supportFragmentManager, null)
                    } catch (e: ClassCastException) {
                        e.printStackTrace()
                    }
                }

            } ?: run {
                suggestBtn.visibility = View.GONE
                suggestBtn.setOnClickListener {}
            }

        }
    }

    private fun updateUpcomingSongButton() {

        val upcomingBtn = findViewById<Button>(R.id.details_btn_upcoming_in_path)

        db.songPathDao().nextAsSong?.let { pathSong ->

            upcomingBtn.visibility = View.VISIBLE
            upcomingBtn.setText(getString(R.string.details_btn_upcoming_inPath, pathSong.title))

            upcomingBtn.setOnClickListener {
                song = pathSong
                db.songPathDao().deleteCurrent()
                updateView()
            }
        } ?: run {
            upcomingBtn.visibility = View.GONE
        }

    }

    private fun randFabClick() {
        val randomSong = db.songDao().loadRandomFilteredByUser()
        if (randomSong == null) {
            SnackbarHelper(findViewById(R.id.songDetails_main_layout)).makeNoBookSnackbar()
        } else {
            song = randomSong
            songOpenedByRandom = true
            updateView()
        }
    }

    private fun updateCategoryPills() {
        song?.let { song ->
            catChipGroup.removeAllViews()
            val pillMaker = CategoryPillMaker(this, db)
            for (category in db.categoryDao().findBySongId(song.id)) {
                catChipGroup.addView(pillMaker.normalPill(category) {
                    pillMaker.normalPillOnClick(it)
                })
            }
            catChipGroup.addView(pillMaker.newPill(song, supportFragmentManager))
        }
    }

    private fun updateSuggestionPills() {

        song?.let { song2 ->
            suggestionChipGroup.removeAllViews()
            val pillMaker = SuggestionPillMaker(this, db)

            for (songSuggestion in db.songSuggestionDao()
                .findAsSongBySongIdBidirectional(song2.id)) {
                suggestionChipGroup.addView(pillMaker.normalPill(songSuggestion) {
                    if (songOpenedByRandom) appendToHistory()
                    song = it
                    songOpenedByRandom = false
                    updateView()
                })
            }
            suggestionChipGroup.addView(pillMaker.newPill(song2, supportFragmentManager))
        }
    }

    private fun updatePages() {

        song?.let { song ->
            val books: MutableList<Book>
            val bookPagesList: MutableList<BookPage> = ArrayList()
            val adapter = PagesAdapter(bookPagesList)
            if (book == null) {
                books = db.bookDao().loadAllByUser().toMutableList()
            } else {
                books = ArrayList()
                books.add(db.bookDao().findById(book!!.id))
            }
            for (book in books) {
                val bookRelation = db.bookRelationDao().getBySongAndBook(song.id, book.id)

                //when song exists in book and book is active
                bookRelation?.let { br ->
                    book.name?.let { bookName ->
                        bookPagesList.add(BookPage(bookName, br.page))
                    }
                }
            }
            pageRecyclerView.adapter = adapter
        }
    }

    private fun updateTexts() {

        song?.let { song ->
            toolbar.title = song.title
            toolbar.subtitle = song.beginning
        }
    }

    private fun appendToHistory() {
        song?.let { song ->
            db.historyDao().addSong(song.id, timestamp)
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) { // User chose the Back Button, show the app settings UI...
            finish()
            return true
        } // If we got here, the user's action was not recognized.
        // Invoke the superclass to handle it.
        return super.onOptionsItemSelected(item)
    }

    public override fun onDestroy() {
        if (initialSong && songOpenedByRandom) appendToHistory()
        super.onDestroy()
    }
}